package pl.elzab.virtualcomsample;

import android.util.Log;
import com.hoho.android.usbserial.util.HexDump;

import pl.elzab.virtualcomsample.dataobject.IngoingResponse;
import pl.elzab.virtualcomsample.protocol.AngelProtocol;
import pl.elzab.virtualcomsample.protocol.ElzabProtocol;
import pl.elzab.virtualcomsample.protocol.StxProtocol;

/**
 * Class used to classify ingoing response.
 * Supported protocols: Elzab, Angel, STX.
 * @author ElzabSoft
 * @version 1.01
 */
public class IngoingResponseHandler {
    /** tag for logging */
    public static final String TAG = "ElzabUSB";
    /** message buffer, here we collect received parts of frame to build a full frame */
    private byte[] mFullResponse = new byte[]{};

    /**
     * classifies received frame (or a part of frame)
     * @param data received frame or a part of frame
     * @return response type
     */
    private IngoingResponse.ResponseType CheckResponseType(byte[] data) {
        if(ElzabProtocol.IsElzabBasicMeasurement(data))
            return IngoingResponse.ResponseType.ELZAB_BASIC_MEASUREMENT;
        if(ElzabProtocol.IsElzabExtendedMeasurement(data))
            return IngoingResponse.ResponseType.ELZAB_EXTENDED_MEASUREMENT;
        if(ElzabProtocol.IsElzabPresence(data))
            return IngoingResponse.ResponseType.ELZAB_PROTOCOL_PRESENCE;
        if(AngelProtocol.IsAngelPresence(data))
            return IngoingResponse.ResponseType.ANGEL_PROTOCOL_PRESENCE;
        if(AngelProtocol.IsAngelMeasurement(data))
            return IngoingResponse.ResponseType.ANGEL_MEASUREMENT;
        if(StxProtocol.IsStxFrame(data))
        {
            byte[] dataField = StxProtocol.GetDataField(mFullResponse);
            if(StxProtocol.IsEmptyFrame(dataField))
                return IngoingResponse.ResponseType.STX_EMPTY_FRAME;
            if(StxProtocol.IsUniqueNumber(dataField))
                return IngoingResponse.ResponseType.STX_UNIQUE_NR;
            if(StxProtocol.IsError(dataField))
                return IngoingResponse.ResponseType.STX_ERROR;
            if(StxProtocol.IsErrorDescription(dataField))
                return IngoingResponse.ResponseType.STX_ERROR_DESCRIPTION;
            if(StxProtocol.IsCodingInfo(dataField))
                return IngoingResponse.ResponseType.STX_CODING_INFO;
            if(StxProtocol.IsDeviceInfo(dataField))
                return IngoingResponse.ResponseType.STX_DEVICE_PRESENCE;
        }
        return IngoingResponse.ResponseType.UNKNOWN;
    }

    /** clears message buffer */
    public void ClearMessageBuffer() {
        mFullResponse = new byte[]{};
    }

    /**
     * processes ingoing not yet classified bytes
     * @param serialOutput ingoing response raw bytes
     * @return IngoingResponse object
     */
    public synchronized IngoingResponse ProcessSerialOutput(byte[] serialOutput) {
        IngoingResponse response = new IngoingResponse();
        if( serialOutput == null) {
            ClearMessageBuffer();
            return null;
        }
        // here we build a full frame, because sometimes single frame comes in multiple parts
        mFullResponse = Util.ConcatenateByteArrays(mFullResponse, serialOutput);
        IngoingResponse.ResponseType type = CheckResponseType(mFullResponse);
        Log.d(TAG, "Processing part of frame: " + HexDump.toHexString(serialOutput));
        Log.d(TAG, "Current full frame: " + HexDump.toHexString(mFullResponse));
        response.SetResponseType(type);
        response.SetResponseBytes(mFullResponse);
        if(type != IngoingResponse.ResponseType.UNKNOWN)
            ClearMessageBuffer();
        Log.d(TAG, "Assigned frame to: " + response.GetResponseType().toString());
        return response;
    }
}
