package pl.elzab.virtualcomsample;

import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbDeviceConnection;
import android.hardware.usb.UsbManager;
import android.util.Log;
import com.hoho.android.usbserial.driver.UsbSerialDriver;
import com.hoho.android.usbserial.driver.UsbSerialPort;
import com.hoho.android.usbserial.driver.UsbSerialProber;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import pl.elzab.virtualcomsample.dataobject.StatusInfo;

/**
 * USB serial connection singleton.
 * @author ElzabSoft
 * @version 1.01
 */
public class USBConnection {
    /** USB serial connection instance */
    private static UsbSerialPort sSerialPort = null;
    /** broadcast receiver for receiving USB permission */
    private static MyBroadcastReceiver sMyBroadcastReceiver;
    /** USB permission intent */
    private static PendingIntent sPermissionIntent;
    /** tag for logging */
    public static final String TAG = "ElzabUSB";

    /**
     * registers broadcast receiver
     * @param context application context
     * @return pending intent
     */
    public static PendingIntent RegisterBroadcastReceiver(Context context) {
        sMyBroadcastReceiver = new MyBroadcastReceiver();
        sPermissionIntent = PendingIntent.getBroadcast(context, 0, new Intent(sMyBroadcastReceiver.ACTION_USB_PERMISSION), 0);
        IntentFilter filter = new IntentFilter(sMyBroadcastReceiver.ACTION_USB_PERMISSION);
        context.registerReceiver(sMyBroadcastReceiver, filter);
        return sPermissionIntent;
    }

    /**
     * gets USB permission intent
     * @param context application context
     * @return USB permission intent
     */
    public static synchronized PendingIntent getPermissionIntent(Context context) {
        if( sPermissionIntent != null)
            return sPermissionIntent;
        return RegisterBroadcastReceiver(context);
    }

    /**
     * initiates USB permission request
     * @param context application context
     * @param usbDevice UsbDevice object
     */
    public static void RequestForPermission(Context context, UsbDevice usbDevice) {
        UsbManager usbManager = (UsbManager) context.getSystemService(Context.USB_SERVICE);
        if( !usbManager.hasPermission(usbDevice) ) {
            usbManager.requestPermission(usbDevice, getPermissionIntent(context));
        }
    }

    /**
     * searches usb device for serial port
     * @param usbDevice UsbDevice object
     * @return USB serial port
     */
    private static UsbSerialPort FindPort(UsbDevice usbDevice) {
        UsbSerialDriver driver = UsbSerialProber.getDefaultProber().probeDevice(usbDevice);
        List<UsbSerialPort> result = new ArrayList<UsbSerialPort>();
        result.addAll(driver.getPorts());
        return result.isEmpty() ? null : result.get(0);
    }

    /**
     * searches for serial port in available drivers
     * @param drivers list of available drivers
     * @return USB serial port
     */
    private static UsbSerialPort FindPort(List<UsbSerialDriver> drivers) {
        final List<UsbSerialPort> result = new ArrayList<UsbSerialPort>();
        for (final UsbSerialDriver driver : drivers) {
            final List<UsbSerialPort> ports = driver.getPorts();
            result.addAll(ports);
        }
        return result.isEmpty() ? null : result.get(0);
    }

    /**
     * gets USB serial port connection instance
     * @param context application context
     * @return USB serial port connection instance
     */
    public static synchronized UsbSerialPort getConnection(Context context) {
        if( sSerialPort != null)
            return sSerialPort;
        UsbManager usbManager = (UsbManager) context.getSystemService(Context.USB_SERVICE);
        final List<UsbSerialDriver> drivers = UsbSerialProber.getDefaultProber().findAllDrivers(usbManager);
        if (drivers.isEmpty()) {
            Util.sendParcelable(context, new StatusInfo(context.getString(R.string.connecting_failed), StatusInfo.MessageStatus.ERROR), "message-for-activity", "status_info");
            Log.d(TAG, "No drivers found.");
            return null;
        }
        sSerialPort = FindPort(drivers);
        if( sSerialPort == null) {
            Util.sendParcelable(context, new StatusInfo(context.getString(R.string.connecting_failed), StatusInfo.MessageStatus.ERROR), "message-for-activity", "status_info");
            Log.d(TAG, "No ports found.");
            return sSerialPort;
        }
        UsbDeviceConnection connection;
        if( usbManager.hasPermission(sSerialPort.getDriver().getDevice())) {
            connection = usbManager.openDevice(sSerialPort.getDriver().getDevice());
        } else {
            Log.d(TAG, "Permission denied for USB device.");
            Util.sendParcelable(context, new StatusInfo(context.getString(R.string.no_usb_permission), StatusInfo.MessageStatus.ERROR), "message-for-activity", "status_info");
            UsbDevice device = sSerialPort.getDriver().getDevice();
            closeConnection(context);
            RequestForPermission(context, device);
            Log.d(TAG, "After requesting permission.");
            return null;
        }
        try {
            sSerialPort.open(connection);
            sSerialPort.setParameters(115200, 8, UsbSerialPort.STOPBITS_1, UsbSerialPort.PARITY_NONE);
        } catch (IOException e) {
            Log.d(TAG, "Serial port opening failed.");
            Util.sendParcelable(context, new StatusInfo(context.getString(R.string.connecting_failed), StatusInfo.MessageStatus.ERROR), "message-for-activity", "status_info");
            closeConnection(context);
            return null;
        }
        Util.sendParcelable(context, new StatusInfo(context.getString(R.string.connection_opened), StatusInfo.MessageStatus.OK), "message-for-activity", "status_info");
        Log.d(TAG, "Connection successfully opened.");
        return sSerialPort;
    }

    /**
     * gets USB serial port connection instance from specified usb device
     * @param context application context
     * @param usbDevice specified USB device
     * @return USB serial port connection instance
     */
    public static synchronized UsbSerialPort getConnection(Context context, UsbDevice usbDevice) {
        UsbDeviceConnection connection;
        UsbManager usbManager = (UsbManager) context.getSystemService(Context.USB_SERVICE);
        if( usbManager.hasPermission(usbDevice) ) {
            connection = usbManager.openDevice(usbDevice);
        } else {
            Log.d(TAG, "Permission denied for USB device.");
            closeConnection(context);
            Util.sendParcelable(context, new StatusInfo(context.getString(R.string.no_usb_permission), StatusInfo.MessageStatus.OK), "message-for-activity", "status_info");
            return null;
        }
        sSerialPort = FindPort(usbDevice);
        try {
            sSerialPort.open(connection);
            sSerialPort.setParameters(115200, 8, UsbSerialPort.STOPBITS_1, UsbSerialPort.PARITY_NONE);
        } catch (IOException e) {
            Log.d(TAG, "Serial port opening failed.");
            Util.sendParcelable(context, new StatusInfo(context.getString(R.string.connecting_failed), StatusInfo.MessageStatus.OK), "message-for-activity", "status_info");
            closeConnection(context);
            return null;
        }
        Util.sendParcelable(context, new StatusInfo(context.getString(R.string.connection_opened), StatusInfo.MessageStatus.OK), "message-for-activity", "status_info");
        Log.d(TAG, "Connection successfully opened.");
        return sSerialPort;
    }

    /**
     * closes USB serial port connection instance
     * @param context application context
     */
    public static synchronized void closeConnection(Context context) {
        try {
            if( sSerialPort != null ) {
                Util.sendParcelable(context, new StatusInfo(context.getString(R.string.connection_closed), StatusInfo.MessageStatus.OK), "message-for-activity", "status_info");
                Log.d(TAG, "Connection closed.");
                sSerialPort.close();
            }
        } catch (IOException e) {
        }
        sSerialPort = null;
    }
}
