package pl.elzab.virtualcomsample.protocol;

import android.util.Log;

import pl.elzab.virtualcomsample.dataobject.Measurement;

/**
 * Class used to generate or parse Elzab protocol frames.
 * @author ElzabSoft
 * @version 1.0
 */
public class ElzabProtocol {
    /** tag for logging */
    public static final String TAG = "ElzabUSB";
    /** scales number hex value representation */
    public enum ScalesNumber {
        SCALES_0(0x0A),
        SCALES_1(0x1A),
        SCALES_2(0x2A),
        SCALES_3(0x3A);

        private int numVal;

        private ScalesNumber(int val) {
            this.numVal = val;
        }

        public byte getValue() {
            return (byte)numVal;
        }
    }

    /** measurement format types */
    public enum MeasurementFormat {
        /** use device-specified measurement format */
        DEVICE_SPECIFIED(0x61),
        /** force basic measurement format */
        FORCE_BASIC(0x71),
        /** force extended measurement format */
        FORCE_EXTENDED(0x81);

        private int numVal;

        private MeasurementFormat(int val) {
            this.numVal = val;
        }

        public byte getValue() {
            return (byte)numVal;
        }
    }

    /**
     * generates byte command to get weight measurement in Elzab protocol
     * @return byte command
     */
    public static byte[] ElzabMeasurement(ScalesNumber scalesNumber, MeasurementFormat measurementFormat) {
        if(scalesNumber == null || measurementFormat == null)
            return null;
        return new byte[] {0x1B, 0x4D, 0x03, measurementFormat.getValue(), scalesNumber.getValue()};
    }

    /**
     * generates byte command to check if device is set to Elzab protocol
     * @return byte command
     */
    public static byte[] ElzabScalesPresence(ScalesNumber scalesNumber) {
        if( scalesNumber == null)
            return null;
        return new byte[] { 0x1B, 0x4D, 0x03, 0x66, scalesNumber.getValue()};
    }

    /**
     * check whether the frame received by device indicates that it's elzab protocol device
     * @param frame received frame
     * @return true if it's elzab protocol device
     */
    public static boolean IsElzabPresence(byte[] frame) {
        return  frame.length == 1 &&  frame[0] == 0x1D;
    }

    /**
     * check whether the frame received by device indicates that it's elzab basic measurement frame
     * @param frame received frame
     * @return true if it's elzab basic measurement frame
     */
    public static boolean IsElzabBasicMeasurement(byte[] frame) {
        return  frame.length == 10 && ( frame[0] == 0x20 ||  frame[0] == 0x2D) &&  frame[1] == 0x20 &&  frame[9] == 0x0A;
    }

    /**
     * check whether the frame received by device indicates that it's elzab extended measurement frame
     * @param frame received frame
     * @return true if it's elzab extended measurement frame
     */
    public static boolean IsElzabExtendedMeasurement(byte[]  frame) {
        return  frame.length == 11 &&  frame[0] == 0x1B && ( frame[1] == 0x53 ||  frame[1] == 0x55) &&  frame[10] == 0x0A;
    }

    /**
     * parses Measurement from Elzab protocol frame
     * @param data frame bytes
     * @return Measurement object
     */
    public static Measurement ParseElzabBasicMeasurement(byte[] data) {
        Measurement measurement = null;
        Log.d(TAG, "Parsing Elzab Basic measurement.");
        if( data.length == 10) {
            measurement = new Measurement();
            byte []array = new byte[10];
            for(int i=0; i<10; i++)
                array[i] = data[i];
            StringBuilder stringbuilder = new StringBuilder();
            for(int i=0; i<6; i++)
                if(array[i+2] != ' ')
                    stringbuilder.append(new String(array, i+2, 1));
            try {
                measurement.SetWeight(Double.parseDouble(stringbuilder.toString()));
                if( data[0] == 0x2D) // minus sign, so negate the value
                    measurement.SetWeight(measurement.GetWeight() * -1.0);
            }
            catch(Exception e) {
                measurement.SetWeight(0.0);
            }
            measurement.SetUnit(Measurement.Unit.KILOGRAM);
            measurement.SetOverflow(false);
            measurement.SetStability(Measurement.Stability.STABLE);
        }
        Log.d(TAG, "Parsing Elzab Basic measurement finished.");
        return measurement;
    }

    /**
     * parses Measurement from Elzab protocol frame (extended measurement data)
     * @param data frame bytes
     * @return Measurement object
     */
    public static Measurement ParseElzabExtendedMeasurement(byte[] data) {
        Measurement measurement = null;
        Log.d(TAG, "Parsing Elzab Extended measurement.");
        if( data.length == 11) {
            measurement = new Measurement();
            byte []array = new byte[11];
            for(int i=0; i<11; i++)
                array[i] = data[i];
            StringBuilder stringbuilder = new StringBuilder();
            for(int i=0; i<6; i++)
                if(array[i+3] != ' ')
                    stringbuilder.append(new String(array, i+3, 1));
            try {
                measurement.SetWeight(Double.parseDouble(stringbuilder.toString()));
                if (data[3] == 0x2D) // minus sign, so negate the value
                    measurement.SetWeight(measurement.GetWeight() * -1.0);
            } catch (Exception e) {
                measurement.SetWeight(0.0);
            }
            measurement.SetUnit(Measurement.Unit.KILOGRAM);
            measurement.SetOverflow(false);
            measurement.SetStability(data[1] == 0x53 ? Measurement.Stability.STABLE : Measurement.Stability.UNSTABLE);
        }
        Log.d(TAG, "Parsing Elzab Extended measurement finished.");
        return measurement;
    }
}
